/*******************************************************************************
 * main_game.c
 *
 * 	Created on : May 9, 2012
 * 	     Author: a0866685
 * 	      Board: DRV2603EVM-CT REVD
 *
 * Description: This program controls the DRV2603EVM-CT evaluation board.
 *      The program demonstrates waveforms with/without braking and with/without
 *      auto-resonance control and ERM waveforms with/without braking.
 * 
 * I/O
 * 		   -------------
 * 		  |			P1.0|--> Red LED
 * 		  |			P1.1|--> Green LED
 * 		  |			P1.2|--> Yellow LED
 *	M0 <--|P3.3		P1.3|--> Blue LED
 * 	M1 <--|P3.4		P1.4|<-- Audio-to-Haptics
 * 	M2 <--|P3.5		P2.0|<-- B1 Button
 * 	M3 <--|P3.6		P2.1|<-- B2 Button
 * 	M4 <--|P3.7  	P2.2|<-- B3 Button
 * 		  |			P2.3|<-- B4 Button
 * 		  |			P2.4|<-- Minus Button
 * SDA <->|			P2.5|<-- Plus button
 * SCL <->|			P2.6|--> Load Switch Control (LRA/ERM Select)
 * 		  |			P3.0|--> DRV2603 LRA/ERM Select
 * 		  |			P3.1|--> DRV2603 Enable
 *		  |			P3.2|--> PWM Out
 *		   -------------
 *
 * *****************************************************************************
 * Button Descriptions 
 * *****************************************************************************
 * Mode Off (All LEDs Off) - Ramps + clicks - ERM vs LRA
 * B1: LRA Ramp up and tick
 * B2: LRA Bump then ramp down
 * B3: ERM Ramp up and then click
 * B4: ERM Bump then ramp down
 * 
 * Mode 0 (M4 On) - Alerts - ERM vs LRA
 * B1: LRA Alert with auto resonance control
 * B2: LRA Alert without auto resonance control
 * B3: ERM Alert
 * B4: Empty - No haptics, just visual alert
 *
 * Mode 1 (M3 On) - Clicks - LRA with auto-resonance on
 * B1: LRA Click with braking (auto-resonance on)
 * B2: LRA Click without braking (auto-resonance on)
 * B3: LRA Double-Click with braking (auto-resonance on)
 * B4: LRA Double-Click without braking (auto-resonance on)
 * 
 * Mode 2 (M2 On) - LRA Keyboard Effects
 * B1: LRA Click
 * B2: LRA Touch & Release Click
 * B3: LRA Warning Click (different style click)
 * B4: LRA Scroll Wheel Effect
 * 
 * Mode 3 (M2 On) - Clicks - ERM Mode
 * B1: ERM Click with braking (hold to repeat)
 * B2: ERM Click without braking (hold to repeat)
 * B3: ERM Double-Click with braking (hold to repeat)
 * B4: ERM Double-Click without braking (hold to repeat)
 *
 * Mode 4 (M4 On) - Concentration - Game where you repeat the pattern
 * 1) Press any button (B1-B4) to start the game
 * 2) The game counts down and then shows a pattern of signals
 * 3) Once you have seen the pattern, repeat the pattern by touching the buttons
 * 		in the same order
 * 4) If you repeat the entire pattern correctly, the board will repeat the
 * 		pattern plus one additional signal.  Continue playing until you repeat
 * 		the sequence incorrectly.
 * 5) If you repeat the pattern incorrectly, the board will buzz and the game
 * 		is over.  The mode LEDs will scroll and then you score will appear in
 * 		binary.
 ******************************************************************************/
#include "msp430g2553.h"
#include "CapTouchBoard.h"
#include "Actuator_Waveforms.h"
#include "SimonGame.h"
#include "BinaryModes.h"

#define SCROLL 250

extern struct Element* modePtr;			// Pointer to current mode button pressed
extern struct Element* buttonPtr;		// Pointer to current effect button pressed
struct Element* lastButtonPtr;			// Pointer to last effect button pressed

const uint8_t buttonLeds[NUMBER_BUTTONS] =
{
  BIT0,
  BIT1,
  BIT2,
  BIT3
};

const struct Element* buttonList[NUMBER_BUTTONS] = 	// Array of CTS elements (buttons)
{
  &button1,
  &button2,
  &button3,
  &button4
};            

unsigned int i;

// Main Function
void main(void)
{ 
  WDTCTL = WDTPW + WDTHOLD;             // Stop watchdog timer
  
  // Initialize MSP430 Clocks
  BCSCTL1 = CALBC1_8MHZ;                // Set DCO to 1, 8, 12 or 16MHz
  DCOCTL = CALDCO_8MHZ;					// Set MODx to 3
  BCSCTL1 |= DIVA_0;                    // ACLK/(0:1,1:2,2:4,3:8)
  BCSCTL2 |= DIVS_3;                    // SMCLK/(0:1,1:2,2:4,3:8)
  BCSCTL3 |= LFXT1S_2;                  // LFXT1 = VLO (select SMCLK source)
  
  // Set GPIO functions
  P2SEL &= 0xBF;						// P2.0-P2.5 = cap sense, P2.6 = I/O
  P2SEL2 &= 0xBF;						// P2.0-P2.5 = cap sense, P2.6 = I/O
  P3SEL |= 0x04;                   		// P3.2 = PWM Output
  
  // Set GPIO directions
  P1DIR |= (BUTTON1+BUTTON2+BUTTON3+BUTTON4);				// Set P1.0-P1.3 to outputs for colored LEDs		
  P2DIR |= 0x40;											// Set P2.6 = Output 
  P3DIR |= (MODE0+MODE1+MODE2+MODE3+MODE4+BIT2+BIT1+BIT0);  // Set P3.0-P3.1, P3.3-P3.7 as outputs = LRA/^ERM, EN, M0-M4
  
  // Set GPIO values
  P1OUT = 0;							// Colored LEDs off
  P2OUT |= 0x40;						// Set P2.6 = 1, set load switch to LRA
  P3OUT = 0x00;                    		// White LEDs off, EN = off
 
  // TimerA Setup
  // Set PWM Frequency,  TimerA gives 255 counts at SMCLK frequency
  TA1CCR0 = 0x00FF;
  
  CapTouch_Init();						// Initialize the cap touch board setting and statuses
  Haptics_Init();						// Initialize Haptics settings

  CapTouch_PowerUpSequence();			// CapTouch Board PowerUp Sequence
  
  // Main loop starts here
  while (1)
  {
    modePtr = (struct Element *)TI_CAPT_Buttons(&mode_change);		// detect mode button press
    buttonPtr = (struct Element *)TI_CAPT_Buttons(&buttons);		// detect effect button press
    
    if(modePtr)		// if a mode button is pressed
    {
    	CapTouch_ModeLEDsOff();				// Reset mode LEDs before next update
    	CapTouch_UnfreezeLEDs();			// Unfreeze LEDs if set to frozen
    	Simon_Reset(); 						// if the mode button is pressed, cancel current Simon game

    	if(CapTouch_isModeBtnReleased)		// if this is first time to register a mode button press
    	{
    		CapTouch_isModeBtnReleased = 0; 	// Do not increment mode again until mode button is released and then pressed again

    		if(modePtr == &mode_up)				// increment mode +1
    		{
    			CapTouch_IncrementMode();
    			CapTouch_EnableOnHold(CapTouch_SetBinaryMode);		// change mode counter style to binary
    		}
    		else
    		{
    			CapTouch_DecrementMode();		// decrement mode -1
    			CapTouch_EnableOnHold(CapTouch_UnsetBinaryMode);		// change mode counter style to carousel
    		}

    		CapTouch_mode &= 0xF8;
    	}
    }
    else			// else no mode button is pressed
    {
    	CapTouch_ModeRepeatReset();				// reset status and counters when repeat mode is finished
    	CapTouch_isModeBtnReleased = 1;			// Mode button has been released
    }

    P3OUT |= CapTouch_mode;						// Set mode LEDs to current mode

    // Manage the colored LEDs
	if(!CapTouch_isLEDFrozen)			// if LEDs are not frozen, reset button LEDs
		CapTouch_ButtonLEDsOff();
    
    if(buttonPtr)		// if an effect button is pressed
    {
		P1OUT |= buttonPtr->referenceNumber;	// Turn on button LED

		lastButtonPtr = buttonPtr;				// Store the button pressed as the lastButton

		if(CapTouch_isEffectBtnReleased)		// if first time button is pressed
		{
			CapTouch_isEffectBtnReleased = 0;	// Disable repeat effect

			if(CapTouch_mode == MODEOFF)		// Mode OFF - All LEDs off
			{
				switch(buttonPtr->referenceNumber)
				{
				case BUTTON1:	// LRA ramp-up, then click
				{
					Haptics_SendWaveform(lra_rampup);
					Haptics_SendWaveform(lra_click);
					break;
				}
				case BUTTON2:	// LRA click, then ramp-down
				{
					Haptics_SendWaveform(lra_click);
					Haptics_SendWaveform(lra_rampdown);
					break;
				}
				case BUTTON3:	// ERM ramp-up, then click
				{
					Haptics_SendWaveform(erm_rampup);
					Haptics_SendWaveform(erm_click);
					break;
				}
				case BUTTON4:	// ERM click, then ramp-down
				{
					Haptics_SendWaveform(erm_click);
					Haptics_SendWaveform(erm_rampdown);
					break;
				}
				default: __no_operation();
				}
				CapTouch_RepeatOnHoldWait();		// Repeat the effect after a short pause while holding
			}
			else if(CapTouch_mode == MODE4)			// Mode 4 (LEDs 10000'b) - Alerts - LRA auto-resonance vs LRA no autores vs ERM
			{
				switch(buttonPtr->referenceNumber)
				{
				case BUTTON1:
				{
					Haptics_SendWaveform(lra_alert);		// LRA alert with auto-resonance on
					break;
				}
				case BUTTON2:
				{
					Haptics_SendWaveform(lra_alert_dumb);	// LRA alert with auto-resonance off
					break;
				}
				case BUTTON3:
				{
					Haptics_SendWaveform(erm_alert);		// ERM alert
					break;
				}
				case BUTTON4:
				{
					CapTouch_FlashModeLEDs(3); 				// Visual alert, no haptics
					break;
				}
				default: __no_operation();
				}
				CapTouch_RepeatOnHoldWait();
			}
			else if(CapTouch_mode == MODE3) 	// Mode 3 (LEDs 01000'b) - LRA Clicks - Brake, no brake, double-click with brake, double-click no brake
			{
				switch(buttonPtr->referenceNumber)
				{
				case BUTTON1:
				{
					Haptics_SendWaveform(lra_click);				// LRA click with brake
					break;
				}
				case BUTTON2:
				{
					Haptics_SendWaveform(lra_click_nobrake);		// LRA click without brake
					break;
				}
				case BUTTON3:
				{
					Haptics_SendWaveform(lra_doubleclick);			// LRA double-click with brake
					break;
				}
				case BUTTON4:
				{
					Haptics_SendWaveform(lra_doubleclick_nobrake);	// LRA double-click without brake
					break;
				}

				default: __no_operation();
				}
				CapTouch_RepeatOnHoldWait();
			}
			else if(CapTouch_mode == MODE2)		// Mode 2 (LEDs 00100'b) - LRA - Sample Keyboard Effects
			{
				switch(buttonPtr->referenceNumber)
				{
				case BUTTON1:		// Basic Click (letter key)
				{
					Haptics_SendWaveform(lra_click);
					CapTouch_RepeatOnHoldWait();
					break;
				}
				case BUTTON2:		// Touch and release click (spacebar)
				{
					Haptics_SendWaveform(lra_click);
					break;
				}

				case BUTTON3:		// Quick double click (backspace)
				{
					Haptics_SendWaveform(lra_tick);
					Haptics_SendWaveform(lra_tick);
					CapTouch_RepeatOnHoldWait();
					break;
				}
				case BUTTON4:		// Scroll effect (scroll bar)
				{
					Haptics_SendWaveform(lra_tick);
					sleep(300);
					Haptics_SendWaveform(lra_tick);
					sleep(300);
					Haptics_SendWaveform(lra_tick);
					sleep(300);
					Haptics_SendWaveform(lra_tick);
					sleep(SCROLL);
					Haptics_SendWaveform(lra_tick);
					sleep(SCROLL);
					Haptics_SendWaveform(lra_tick);
					sleep(SCROLL);
					Haptics_SendWaveform(lra_tick);
					sleep(SCROLL);
					Haptics_SendWaveform(lra_tick);
					sleep(SCROLL);
					Haptics_SendWaveform(lra_tick);
					sleep(SCROLL);
					Haptics_SendWaveform(lra_tick);
					sleep(SCROLL);
					Haptics_SendWaveform(lra_tick);
					sleep(SCROLL+50);
					Haptics_SendWaveform(lra_tick);
					sleep(SCROLL+50);
					Haptics_SendWaveform(lra_tick);
					sleep(SCROLL+100);
					Haptics_SendWaveform(lra_tick);
					sleep(SCROLL+100);
					Haptics_SendWaveform(lra_tick);
					sleep(SCROLL+200);
					Haptics_SendWaveform(lra_tick);
					sleep(SCROLL+200);
					Haptics_SendWaveform(lra_tick);
					sleep(SCROLL+200);
					Haptics_SendWaveform(lra_tick);
					sleep(SCROLL+300);
					Haptics_SendWaveform(lra_tick);
					sleep(SCROLL+300);
					Haptics_SendWaveform(lra_tick);
					sleep(SCROLL+400);
					Haptics_SendWaveform(lra_tick);
					sleep(SCROLL+500);
					Haptics_SendWaveform(lra_tick);
					sleep(SCROLL+600);
					Haptics_SendWaveform(lra_tick);
					sleep(SCROLL+700);
					Haptics_SendWaveform(lra_tick);
					break;
				}
				default: __no_operation();
				}
			}
			else if(CapTouch_mode == MODE1)   // Mode 1 (LEDs 00010'b) - ERM Clicks - Brake, no brake, double-click with brake, double-click no brake
			{
				switch(buttonPtr->referenceNumber)
				{
				case BUTTON1:
				{
					Haptics_SendWaveform(erm_click);		// Brake
					break;
				}
				case BUTTON2:
				{
					Haptics_SendWaveform(erm_bump);			// No brake
					break;
				}

				case BUTTON3:
				{
					Haptics_SendWaveform(erm_doubleclick);	// Double-click with brake
					break;
				}
				case BUTTON4:
				{
					Haptics_SendWaveform(erm_doublebump);	// Double-click without brake
					break;
				}
				default: __no_operation();
				}
				CapTouch_RepeatOnHoldWait();
			}
			else if(CapTouch_mode == MODE0)		// Mode 0 (LEDs 00001'b) - Concentration game
			{
				Simon_Play(buttonPtr->sequenceNumber);			// begin Concentration game
			}
			else  // "Additional Binary Modes" - hold the "+" mode button for 3 seconds to enter, hold the "-" mode button to exit
			{
#ifdef BINARYMODES_H_
				/* Include extra binary modes - life test, external PWM mode, etc. */
				BinaryModes();
#endif
			}
		}
    }
    else
    {
    	if(CapTouch_mode == MODE2 && !CapTouch_isEffectBtnReleased) 	// Up button effects (played when the button is released
		{
			switch(lastButtonPtr->referenceNumber)
			{
			case BUTTON1:
			{
				__no_operation();
				break;
			}
			case BUTTON2:
			{
				Haptics_SendWaveform(lra_tick);
				break;
			}
			case BUTTON3:
			{
				__no_operation();
				break;
			}
			case BUTTON4:
			{
				__no_operation();
				break;
			}
			default: __no_operation();
			}
		}

    	CapTouch_RepeatReset();					// reset status and counters when repeat mode is finished
    	CapTouch_isEffectBtnReleased = 1;		// button was released
    }

    sleep(DELAY);
  }
} // End Main

/******************************************************************************/
// Timer0_A0 Interrupt Service Routine:       
/******************************************************************************/
#pragma vector=TIMER0_A0_VECTOR
__interrupt void ISR_Timer0_A0(void)
{
  TA0CTL &= ~(MC_1);
  TA0CCTL0 &= ~(CCIE);

  CapTouch_RandomNumber++;

  __bic_SR_register_on_exit(LPM0_bits+GIE);
}


#pragma vector=TIMER1_A1_VECTOR
__interrupt void Timer_A1_ISR (void)
{
  TA1CCTL1 &= ~CCIFG;   //Clear Interrupt Flag
}
